body > .container {
  background: white;
  padding: 20pt;
  padding-bottom: 100pt;
  border: 1pt solid darkGray;
  position: relative;
  top: 5pt;
}

body {
  background: lightgray;
  font-family: "Book Antiqua", Palatino, "Palatino Linotype", "Palatino LT STD", Georgia, serif;
  /*font-family: Palatino, "Palatino Linotype", "Palatino LT STD", "Book Antiqua", Georgia, serif;*/
}

.page-contents img {
  margin: 0px auto;
  display: block;
}

.label-info {
  cursor: move;
}
